@@ -21,6 +21,7 @@ import networkx as nx
 import numpy as np
 import pandas as pd
 import logging
+import warnings
 
 logger = logging.getLogger(__name__)
 
@@ -209,148 +210,119 @@ def valve_segments(G, valve_layer):
         Number of nodes and links in each segment. The DataFrame is indexed by 
         segment number, with columns named 'node' and 'link'.
     """
+    
+
+    # First check for duplicate valves
+    if valve_layer.duplicated().any():
+        valve_layer.drop_duplicates(inplace = True)
+        warnings.warn('One or more valves were duplicated in `valve_layer`; duplicates are ignored.', stacklevel=0)
+
     # Convert the graph to an undirected graph
     uG = G.to_undirected()
-    
+
     # Node and link names
-    nodes = list(uG.nodes()) # list of node names
-    links = list(uG.edges(keys=True)) # list of tuples with start node, end node, link name
-    
-    # Append N_ and L_ to node and link names, used in matrices
-    matrix_node_names = ['N_'+n for n in nodes]
-    matrix_link_names = ['L_'+k for u,v,k in links]
+    # Append N_ and L_ to node and link names to account for naming overlap
+    node_names = ['N_'+n for n in uG.nodes()]
+    link_names = ['L_'+k for u,v,k in uG.edges(keys=True)]
+    all_names = node_names + link_names
 
-    # Pipe-node connectivity matrix
-    AC = nx.incidence_matrix(uG).astype('int').todense().T
-    AC = pd.DataFrame(AC, columns=matrix_node_names, index=matrix_link_names, dtype=int)
-    
-    # Valve-node connectivity matrix
-    VC = pd.DataFrame(0, columns=matrix_node_names, index=matrix_link_names)
-    for i, row in valve_layer.iterrows():
-        VC.at['L_'+row['link'], 'N_'+row['node']] = 1
-    
-    # Valve deficient matrix (anti-valve matrix)
-    VD = AC - VC
-    del AC, VC
-    
-    # Direct connectivity matrix
-    NI = pd.DataFrame(np.identity(len(matrix_node_names)),
-                      index = matrix_node_names, columns = matrix_node_names)
-    LI = pd.DataFrame(np.identity(len(matrix_link_names)),
-                      index = matrix_link_names, columns = matrix_link_names)
-    DC_left = pd.concat([NI, VD], sort=False)
-    DC_right = pd.concat([VD.T, LI], sort=False)
-    del LI, VD
-    DC = pd.concat([DC_left, DC_right], axis=1, sort=False)
-    del DC_left, DC_right
-    DC = DC.astype(int)
-
-    # initialization for looping routine
+    # Initialization for labelling
     seg_index = 0
-    
-    # pre-processing to find isolated elements before looping
-    seg_label = {}
+    seg_label = np.zeros(shape=(len(all_names)), dtype=int)
 
-    for start_node, end_node, link_name in links:
+    # Find and label links isolated by valves, EG 0|----|0
+    for start_node, end_node, link_name in uG.edges(keys=True):
         link_valves = valve_layer[valve_layer['link']==link_name]
         if set(link_valves['node']) >= set([start_node, end_node]):
             seg_index += 1
-            seg_label['L_'+link_name] = seg_index
-            
-    for node_name in nodes:
+            seg_label[all_names.index('L_'+link_name)] = seg_index
+
+    # Find and label nodes isolated by valves, EG 0----|0|----0
+    for node_name in node_names:
         node_valves = valve_layer[valve_layer['node']==node_name]
-        node_links = [k for u,v,k in uG.edges(node_name, keys=True)]
+        node_links = [k for u,v,k in uG.edges(node_name[2:], keys=True)]
         if set(node_valves['link']) >= set(node_links):
             seg_index += 1
-            seg_label['N_'+node_name] = seg_index
+            seg_label[all_names.index(node_name)] = seg_index
     
-    # drop previously found isolated elements before looping
-    DC = DC.drop(seg_label.keys())
-    DC = DC.drop(seg_label.keys(), axis=1)   
-    
-    DC_np = DC.to_numpy() # requires Pandas v.0.24.0
+    # Collect valved link names
+    valved_link_names = []    
+    for i, row in valve_layer.iterrows():
+        valved_link_names.append(row[0])
 
-    # vector of length nodes+links where the ith entry is the segment number of node/link i
-    seg_label_DC = np.zeros(shape=(len(DC.index)), dtype=int)
+    # Remove valved edges from G
+    valved_edges = []
+    for edge in uG.edges:
+        link_name = edge[2]
+        if link_name in valved_link_names:
+            valved_edges.append(edge)
+    uG.remove_edges_from(valved_edges)
 
-    # Loop over all nodes and links to grow segments
-    for i in range(len(seg_label_DC)):
-        
-        # Only assign a seg_label if node/link doesn't already have one
-        if seg_label_DC[i] == 0:
-            
-            # Advance segment label and assign to node/link, mark as assigned
-            seg_index += 1
-            seg_label_DC[i] = seg_index
-           
-            # Initialize segment size
-            seg_size = (seg_label_DC == seg_index).sum()
-             
-            flag = True
-            
-            #print(i)
-    
-            # Nodes and links that are part of the segment
-            seg = np.where(seg_label_DC == seg_index)[0]
-        
-            # Connectivity of the segment
-            seg_DC = np.zeros(shape=(DC_np.shape[0]), dtype=int)
-            seg_DC[seg] = 1
-    
-            while flag:          
-               
-                # Potential connectivity of the segment      
-                p_seg_DC = DC_np + seg_DC[:,None] # this is slow
-                
-                # Nodes and links that are connected to the segment
-                temp = np.max(p_seg_DC,axis=0) # this is somewhat slow
-                connected_to_seg = np.where(temp > 1)[0]   
-                seg_DC[connected_to_seg] = 1
-      
-                # Label nodes/links connected to the segment
-                seg_label_DC[connected_to_seg] = seg_index
-                
-                # Find new segment size
-                new_seg_size = (seg_label_DC == seg_index).sum()
-                    
-                # Check for progress
-                if seg_size == new_seg_size:
-                    flag = False
-                else:
-                    seg_size = new_seg_size
-          
-                # Update seg_DC and DC_np
-                seg_DC = DC_np[:,i] + np.sum(
-                                        p_seg_DC[:,connected_to_seg],axis=1
-                                        ) # this is slow
-                seg_DC = np.clip(seg_DC,0,1)          
-                DC_np[:,connected_to_seg] = np.repeat(
-                                            seg_DC,len(connected_to_seg)).reshape(
-                                            len(seg_DC),len(connected_to_seg))
-                                            # this is somewhat slow
-    
-    #        print(i, seg_size)
-    
-    # combine pre-processed and looped results
-    seg_labels = list(seg_label.values()) + list(seg_label_DC)
-    seg_labels_index = list(seg_label.keys()) + list(DC.index)
-    seg_label = pd.Series(seg_labels, index=seg_labels_index, dtype=int)
+    ## Label unvalved portion of graph using connected components
 
-    # Separate node and link segments
-    # remove leading N_ and L_ from node and link names
-    node_segments = seg_label[matrix_node_names]
-    link_segments = seg_label[matrix_link_names]
+    # Assign labels to nodes
+    for component in nx.connected_components(uG):
+        # component is a set of node names
+        seg_index += 1
+        for node in component:
+            index = all_names.index('N_'+node)
+            seg_label[index] = seg_index
+
+    # Assign labels to links based on labelling of their nodes
+    for edge in uG.edges(keys=True):
+        node1, node2, link_name = edge
+        node1 = all_names.index('N_'+node1)
+        link_index = all_names.index('L_'+link_name)
+        seg_label[link_index] = seg_label[node1]
+
+    ## Label valved portion of graph
+    for valved_edge in valved_edges:
+        node1_name, node2_name, link_name = valved_edge
+        link_valves = valve_layer[valve_layer['link']==link_name]
+        link_index = all_names.index('L_'+link_name)
+
+        # When link only has one valved node, locate unvalved node
+        # and label link and unvalved node together
+        if link_valves.shape[0] == 1:
+            both_node_names = [node1_name, node2_name]
+            valved_node_name = link_valves.iloc[0][1]
+            both_node_names.remove(valved_node_name)
+            unvalved_node_name = both_node_names[0]
+            unvalved_node_index = all_names.index('N_'+unvalved_node_name)
+            if seg_label[unvalved_node_index] == 0:
+                seg_index += 1
+                seg_label[unvalved_node_index] = seg_index
+                seg_label[link_index] = seg_index
+            else:
+                seg_label[link_index] = seg_label[unvalved_node_index]
+
+        # Links with link_valves.size == 2 are already labelled (isolated link)
+        elif link_valves.shape[0] ==2:
+            continue
+        else:
+            raise Exception("Each link should have a maximum of two valves.")
+
+
+    # Finalize results
+    seg_labels_index = all_names
+    seg_label = pd.Series(seg_label, index=seg_labels_index, dtype=int)
+
+    node_segments = seg_label[node_names]
+    link_segments = seg_label[link_names]
+
+    # Remove "N_" and "L_" from nodes and links
     node_segments.index = node_segments.index.str[2::]
-    link_segments.index = link_segments.index.str[2::]  
-    
+    link_segments.index = link_segments.index.str[2::]
+
     # Extract segment sizes, for nodes and links
     seg_link_sizes = link_segments.value_counts().rename('link')
     seg_node_sizes = node_segments.value_counts().rename('node')
     seg_sizes = pd.concat([seg_link_sizes, seg_node_sizes], axis=1).fillna(0)
     seg_sizes = seg_sizes.astype(int)
-    
+
     return node_segments, link_segments, seg_sizes
 
+
 def valve_segment_attributes(valve_layer, node_segments, link_segments, 
                              demand=None, length=None):
     """
