@@ -1,9 +1,9 @@
 # coding: utf-8
 
 """
-The wntr.network.elements module includes elements of a water network model, 
-including junction, tank, reservoir, pipe, pump, valve, pattern, timeseries, 
-demands, curves, and sources.
+The wntr.network.io module includes functions that convert the water network
+model to other data formats, create a water network model from file, and write 
+the water network model to a file.
 
 .. rubric:: Contents
 
@@ -11,34 +11,51 @@ demands, curves, and sources.
 
     to_dict
     from_dict
+    to_gis
+    from_gis
+    to_graph
     write_json
     read_json
     write_inpfile
     read_inpfile
+    write_geojson
+    read_geojson
+    write_shapefile
+    read_shapefile
 
 """
 import logging
 import json
+import networkx as nx
 
 import wntr.epanet
 from wntr.epanet.util import FlowUnits
 import wntr.network.model
-
+from wntr.gis.network import WaterNetworkGIS
+try:
+    import geopandas as gpd
+    has_geopandas = True
+except ModuleNotFoundError:
+    gpd = None
+    has_geopandas = False
+    
 logger = logging.getLogger(__name__)
 
 
 def to_dict(wn) -> dict:
-    """Dictionary representation of the WaterNetworkModel.
+    """
+    Convert a WaterNetworkModel into a dictionary
     
     Parameters
     ----------
     wn : WaterNetworkModel
-        the water network model to convert to a dictionary
+        Water network model
 
     Returns
     -------
     dict
-        dictionary representation of the water network model
+        Dictionary representation of the WaterNetworkModel
+        
     """
     from wntr import __version__
 
@@ -63,23 +80,22 @@ def to_dict(wn) -> dict:
     return d
 
 
-def from_dict(
-    d: dict, append=None,
-):
+def from_dict(d: dict, append=None):
     """
-    Create or append a water network model object from a dictionary.
+    Create or append a WaterNetworkModel from a dictionary
 
     Parameters
     ----------
     d : dict
-        dictionary representation of the water network model
-    append : WaterNetworkModel, optional
-        an existing water network model to append results to
+        Dictionary representation of the water network model
+    append : WaterNetworkModel or None, optional
+        Existing WaterNetworkModel to append.  If None, a new WaterNetworkModel 
+        is created.
 
     Returns
     -------
     WaterNetworkModel
-        a new water network model or the appended existing model
+    
     """
     from wntr.epanet.io import _read_control_line, _EpanetRule
 
@@ -108,9 +124,7 @@ def from_dict(
         wn.options.__init__(**d["options"])
     if "curves" in d:
         for curve in d["curves"]:
-            wn.add_curve(
-                name=curve["name"], curve_type=curve["curve_type"], xy_tuples_list=curve["points"]
-            )
+            wn.add_curve(name=curve["name"], curve_type=curve["curve_type"], xy_tuples_list=curve["points"])
     if "patterns" in d:
         for pattern in d["patterns"]:
             wn.add_pattern(name=pattern["name"], pattern=pattern["multipliers"])
@@ -142,6 +156,9 @@ def from_dict(
                 j.pressure_exponent = node.setdefault("pressure_exponent")
                 j.required_pressure = node.setdefault("required_pressure")
                 j.tag = node.setdefault("tag")
+                # custom additional attributes
+                for attr in list(set(node.keys()) - set(dir(j))):
+                    setattr( j, attr, node[attr] )
                 if dl is not None and len(dl) > 1:
                     for i in range(1, len(dl)):
                         base_val = dl[i].setdefault("base_val", 0.0)
@@ -170,6 +187,9 @@ def from_dict(
                     t.mixing_model = node.setdefault("mixing_model")
                 t.tag = node.setdefault("tag")
                 t.bulk_coeff = node.setdefault("bulk_coeff")
+                # custom additional attributes
+                for attr in list(set(node.keys()) - set(dir(t))):
+                    setattr( t, attr, node[attr] )
             elif node["node_type"] == "Reservoir":
                 wn.add_reservoir(
                     name,
@@ -180,6 +200,9 @@ def from_dict(
                 r = wn.get_node(name)
                 r.initial_quality = node.setdefault("initial_quality", 0.0)
                 r.tag = node.setdefault("tag")
+                # custom additional attributes
+                for attr in list(set(node.keys()) - set(dir(r))):
+                    setattr( r, attr, node[attr] )
             else:
                 raise ValueError("Illegal node type '{}'".format(node["node_type"]))
     if "links" in d:
@@ -202,6 +225,9 @@ def from_dict(
                 p.tag = link.setdefault("tag")
                 p.vertices = link.setdefault("vertices", list())
                 p.wall_coeff = link.setdefault("wall_coeff")
+                # custom additional attributes
+                for attr in list(set(link.keys()) - set(dir(p))):
+                    setattr( p, attr, link[attr] )
             elif link["link_type"] == "Pump":
                 pump_type = link.setdefault("pump_type", "POWER")
                 wn.add_pump(
@@ -223,6 +249,9 @@ def from_dict(
                 p.initial_setting = link.setdefault("initial_setting")
                 p.tag = link.setdefault("tag")
                 p.vertices = link.setdefault("vertices", list())
+                # custom additional attributes
+                for attr in list(set(link.keys()) - set(dir(p))):
+                    setattr( p, attr, link[attr] )
             elif link["link_type"] == "Valve":
                 valve_type = link["valve_type"]
                 wn.add_valve(
@@ -238,6 +267,9 @@ def from_dict(
                 v = wn.get_link(name)
                 if valve_type.lower() == "gpv":
                     v.headloss_curve_name = link.setdefault("headloss_curve_name")
+                # custom additional attributes
+                for attr in list(set(link.keys()) - set(dir(v))):
+                    setattr( v, attr, link[attr] )
             else:
                 raise ValueError("Illegal link type '{}'".format(link["link_type"]))
     if "sources" in d:
@@ -287,9 +319,115 @@ def from_dict(
     return wn
 
 
-def write_json(
-    wn, path_or_buf, **kw_json,
-):
+def to_gis(wn, crs=None, pumps_as_points=False, valves_as_points=False):
+    """
+    Convert a WaterNetworkModel into GeoDataFrames
+    
+    Parameters
+    ----------
+    wn : WaterNetworkModel
+        Water network model
+    crs : str, optional
+        Coordinate reference system, by default None
+    pumps_as_points : bool, optional
+        Represent pumps as points (True) or lines (False), by default False
+    valves_as_points : bool, optional
+        Represent valves as points (True) or lines (False), by default False
+        
+    Returns
+    -------
+    WaterNetworkGIS object that contains GeoDataFrames
+        
+    """
+    gis_data = WaterNetworkGIS()
+    gis_data._create_gis(wn, crs, pumps_as_points, valves_as_points)
+    
+    return gis_data
+
+
+def from_gis(gis_data, append=None):
+    """
+    Create or append a WaterNetworkModel from GeoDataFrames
+    
+    Parameters
+    ----------
+    gis_data : WaterNetworkGIS or dictionary of GeoDataFrames
+        GeoDataFrames containing water network attributes. If gis_data is a 
+        dictionary, then the keys are junctions, tanks, reservoirs, pipes, 
+        pumps, and valves. If the pumps or valves are Points, they will be 
+        converted to Lines with the same start and end node location.
+    append : WaterNetworkModel or None, optional
+        Existing WaterNetworkModel to append.  If None, a new WaterNetworkModel 
+        is created.
+        
+    Returns
+    -------
+    WaterNetworkModel
+        
+    """
+    if isinstance(gis_data, dict):
+        gis_data = WaterNetworkGIS(gis_data)
+
+    wn = gis_data._create_wn(append=append)
+    
+    return wn
+
+def to_graph(wn, node_weight=None, link_weight=None, modify_direction=False):
+    """
+    Convert a WaterNetworkModel into a networkx MultiDiGraph
+    
+    Parameters
+    ----------
+    node_weight :  dict or pandas Series (optional)
+        Node weights
+    link_weight : dict or pandas Series (optional)
+        Link weights.  
+    modify_direction : bool (optional)
+        If True, than if the link weight is negative, the link start and 
+        end node are switched and the abs(weight) is assigned to the link
+        (this is useful when weighting graphs by flowrate). If False, link 
+        direction and weight are not changed.
+        
+    Returns
+    --------
+    networkx MultiDiGraph
+    """
+    G = nx.MultiDiGraph()
+
+    for name, node in wn.nodes():
+        G.add_node(name)
+        nx.set_node_attributes(G, name="pos", values={name: node.coordinates})
+        nx.set_node_attributes(G, name="type", values={name: node.node_type})
+
+        if node_weight is not None:
+            try:  # weight nodes
+                value = node_weight[name]
+                nx.set_node_attributes(G, name="weight", values={name: value})
+            except:
+                pass
+
+    for name, link in wn.links():
+        start_node = link.start_node_name
+        end_node = link.end_node_name
+        G.add_edge(start_node, end_node, key=name)
+        nx.set_edge_attributes(G, name="type", values={(start_node, end_node, name): link.link_type})
+
+        if link_weight is not None:
+            try:  # weight links
+                value = link_weight[name]
+                if modify_direction and value < 0:  # change the direction of the link and value
+                    G.remove_edge(start_node, end_node, name)
+                    G.add_edge(end_node, start_node, name)
+                    nx.set_edge_attributes(G, name="type", values={(end_node, start_node, name): link.link_type})
+                    nx.set_edge_attributes(G, name="weight", values={(end_node, start_node, name): -value})
+                else:
+                    nx.set_edge_attributes(G, name="weight", values={(start_node, end_node, name): value})
+            except:
+                pass
+    
+    return G
+
+def write_json(wn, path_or_buf, **kw_json,):
     """
     Write the WaterNetworkModel to a JSON file
 
@@ -298,7 +436,8 @@ def write_json(
     path_or_buf : str or IO stream
         Name of the file or file pointer
     kw_json : keyword arguments
-        arguments to pass directly to `json.dump`
+        Arguments to pass directly to `json.dump`
+        
     """
     if isinstance(path_or_buf, str):
         with open(path_or_buf, "w") as fout:
@@ -307,22 +446,24 @@ def write_json(
         json.dump(to_dict(wn), path_or_buf, **kw_json)
 
 
-def read_json(
-    path_or_buf, append=None, **kw_json,
-):
+def read_json(path_or_buf, append=None, **kw_json):
     """
-    Create a WaterNetworkModel from a JSON file.
+    Create or append a WaterNetworkModel from a JSON file
 
     Parameters
     ----------
     f : str
         Name of the file or file pointer
+    append : WaterNetworkModel or None, optional
+        Existing WaterNetworkModel to append.  If None, a new WaterNetworkModel 
+        is created.
     kw_json : keyword arguments
-        keyword arguments to pass to `json.load`
+        Keyword arguments to pass to `json.load`
 
     Returns
     -------
     WaterNetworkModel
+    
     """
     if isinstance(path_or_buf, str):
         with open(path_or_buf, "r") as fin:
@@ -332,11 +473,10 @@ def read_json(
     return from_dict(d, append)
 
 
-def write_inpfile(
-    wn, filename: str, units=None, version: float = 2.2, force_coordinates: bool = False,
-):
+def write_inpfile(wn, filename: str, units=None, version: float = 2.2, 
+                  force_coordinates: bool = False):
     """
-    Writes the current water network model to an EPANET INP file
+    Write the WaterNetworkModel to an EPANET INP file
 
     .. note::
 
@@ -346,21 +486,16 @@ def write_inpfile(
         the user to force EPANET 2.0 compatibility at the expense of pressured-dependent analysis 
         options being turned off.
 
-
     Parameters
     ----------
-    wn : WaterNetworkModel
-        model object to write INP file for
-
+    wn : wntr WaterNetworkModel
+        Water network model
     filename : string
         Name of the inp file.
-
     units : str, int or FlowUnits
         Name of the units being written to the inp file.
-
     version : float, {2.0, **2.2**}
         Optionally specify forcing EPANET 2.0 compatibility.
-
     force_coordinates : bool
         This only applies if `self.options.graphics.map_filename` is not `None`,
         and will force the COORDINATES section to be written even if a MAP file is
@@ -373,24 +508,152 @@ def write_inpfile(
         wn._inpfile = wntr.epanet.InpFile()
     if units is None:
         units = wn._options.hydraulic.inpfile_units
-    wn._inpfile.write(
-        filename, wn, units=units, version=version, force_coordinates=force_coordinates
-    )
+    wn._inpfile.write(filename, wn, units=units, version=version, force_coordinates=force_coordinates)
 
 
-def read_inpfile(
-    filename, append=None,
-):
+def read_inpfile(filename, append=None):
     """
-    Defines water network model components from an EPANET INP file
+    Create or append a WaterNetworkModel from an EPANET INP file
 
     Parameters
     ----------
     filename : string
         Name of the INP file.
-
+    append : WaterNetworkModel or None, optional
+        Existing WaterNetworkModel to append.  If None, a new WaterNetworkModel 
+        is created.
+        
+    Returns
+    -------
+    WaterNetworkModel
+    
     """
     inpfile = wntr.epanet.InpFile()
     wn = inpfile.read(filename, wn=append)
     wn._inpfile = inpfile
+    
+    return wn
+
+
+def write_geojson(wn, prefix: str, crs=None, pumps_as_points=True, 
+                  valves_as_points=True):
+    """
+    Write the WaterNetworkModel to a set of GeoJSON files, one file for each
+    network element.
+
+    The GeoJSON only includes information from the water network model.
+    To add results of a simulation or analysis, do:
+
+    .. code::
+
+        wn_gis = wn.to_gis()
+        wn_gis.add_node_attributes(some_data_to_add, 'name_of_attribute')
+        wn_gis.write_geojson(...)
+
+    Parameters
+    ----------
+    wn : wntr WaterNetworkModel
+        Water network model
+    prefix : str
+        File prefix
+    crs : str, optional
+        Coordinate reference system, by default None
+    pumps_as_points : bool, optional
+        Represent pumps as points (True) or lines (False), by default False
+    valves_as_points : bool, optional
+        Represent valves as points (True) or lines (False), by default False
+        
+    """
+    wn_gis = wn.to_gis(crs, pumps_as_points=pumps_as_points, 
+                       valves_as_points=valves_as_points)
+    wn_gis.write_geojson(prefix=prefix)
+
+
+def read_geojson(files, index_col='index', append=None):
+    """
+    Create or append a WaterNetworkModel from GeoJSON files
+
+    Parameters
+    ----------
+    files : dictionary
+        Dictionary of GeoJSON filenames, where the keys are in the set 
+        ('junction', 'tanks', 'reservoirs', 'pipes', 'pumps', 'valves') and 
+        values are the corresponding GeoJSON filename
+    index_col : str, optional
+        Column that contains the element name
+    append : WaterNetworkModel or None, optional
+        Existing WaterNetworkModel to append.  If None, a new WaterNetworkModel 
+        is created.
+
+    Returns
+    -------
+    WaterNetworkModel
+
+    """
+    gis_data = WaterNetworkGIS()
+    gis_data.read_geojson(files, index_col=index_col)
+    wn = gis_data._create_wn(append=append)
+
+    return wn
+
+
+def write_shapefile(wn, prefix: str, crs=None, pumps_as_points=True,
+                    valves_as_points=True):
+    """
+    Write the WaterNetworkModel to a set of ESRI Shapefiles, one directory for
+    each network element.
+
+    The Shapefiles only includes information from the water network model.
+    To add results of a simulation or analysis, do:
+
+    .. code::
+
+        wn_gis = wn.to_gis()
+        wn_gis.add_node_attributes(some_data_to_add, 'name_of_attribute')
+        wn_gis.write_shapefile(...)
+
+    Parameters
+    ----------
+    wn : wntr WaterNetworkModel
+        Water network model
+    prefix : str
+        File and directory prefix
+    crs : str, optional
+        Coordinate reference system, by default None
+    pumps_as_points : bool, optional
+        Represent pumps as points (True) or lines (False), by default False
+    valves_as_points : bool, optional
+        Represent valves as points (True) or lines (False), by default False
+
+    """
+    wn_gis = wn.to_gis(crs, pumps_as_points=pumps_as_points,
+                       valves_as_points=valves_as_points)
+    wn_gis.write_shapefile(prefix=prefix)
+
+def read_shapefile(files, index_col='index', append=None):
+    """
+
+    Create or append a WaterNetworkModel from ESRI Shapefiles
+
+    Parameters
+    ----------
+    files : dictionary
+        Dictionary of Shapefile file or directory names, where the keys are in the set 
+        ('junction', 'tanks', 'reservoirs', 'pipes', 'pumps', 'valves') and 
+        values are the corresponding Shapefile filenames or directories
+    index_col : str, optional
+        Column that contains the element name
+    append : WaterNetworkModel or None, optional
+        Existing WaterNetworkModel to append.  If None, a new WaterNetworkModel 
+        is created.
+
+    Returns
+    -------
+    WaterNetworkModel
+
+    """
+    gis_data = WaterNetworkGIS()
+    gis_data.read_shapefile(files, index_col='index')
+    wn = gis_data._create_wn(append=append)
+
     return wn
